local assets =
{
	Asset("ANIM", "anim/bearger_build.zip"),
    Asset("ANIM", "anim/bearger_basic.zip"),
    Asset("ANIM", "anim/bearger_actions.zip"),
    Asset("SOUND", "sound/bearger.fsb"),
	Asset("IMAGE", "images/friendlybearger.tex"),
	Asset("ATLAS", "images/friendlybearger.xml"),
}

local prefabs =
{
    "groundpound_fx",
    "groundpoundring_fx",
    "bearger_fur",
    "collapse_small",
}

SetSharedLootTable( 'bearger',
{
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},
    {'bearger_fur',      1.00},
})

local BASE_TAGS = {"structure"}
local SEE_STRUCTURE_DIST = 20

local TARGET_DIST = 7.5

local function LeaveWorld(inst)
    inst:Remove()
end

local function NearPlayerBase(inst)
    local pt = inst:GetPosition()
    local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, SEE_STRUCTURE_DIST, BASE_TAGS)
    if #ents >= 2 then
        inst.SeenBase = true
        return true
    end
end

local function OnKeepTarget(inst, target)
    return inst.components.combat:CanTarget(target) 
end

local function NormalRetarget(inst)
    local notags = {"FX", "NOCLICK","INLIMBO", "companion", "character", "player", "structure"}
    return FindEntity(inst, 16, 
        function(guy) 
            if inst.components.combat:CanTarget(guy) then
                return (guy:HasTag("monster") or guy:HasTag("hostile") or guy.components.combat.target == GetPlayer() or GetPlayer().components.combat.target == guy)
            end
    end, nil, notags)
end

local function ShouldSleep(inst)
    return false
end

local function ShouldWake(inst)
    return true
end

local function OnNewTarget(inst, data)
    inst.components.combat:ShareTarget(data.target, 30, function(dude) return dude:HasTag("summonedbyplayer") end, 15)
end

local function OnAttacked(inst, data)
    local attacker = data.attacker
    inst.components.combat:SetTarget(attacker)
    inst.components.combat:ShareTarget(attacker, 30, function(dude) return dude:HasTag("summonedbyplayer") end, 15)
end

local function WorkEntities(inst)
    local pt = inst:GetPosition()
    local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, 5)
    local heading_angle = -(inst.Transform:GetRotation())
    local dir = Vector3(math.cos(heading_angle*DEGREES),0, math.sin(heading_angle*DEGREES))

    for k,v in pairs(ents) do
        if v and v.components.workable and not v:HasTag("insect") and (v.components.workable.action == ACTIONS.CHOP or v.components.workable.action == ACTIONS.MINE) then
            local hp = v:GetPosition()
            local offset = (hp - pt):GetNormalized()     
            local dot = offset:Dot(dir)
            if dot > .3 then
                v.components.workable:Destroy(inst)
            end
        end
    end
end

local function OnCollide(inst, other)
    if not inst.sg:HasStateTag("running") and not other:HasTag("tree") then return end
    
    local v1 = Vector3(inst.Physics:GetVelocity())
    if v1:LengthSq() < 1 then return end

    inst:DoTaskInTime(2*FRAMES, function()
        if other and other.components.workable and other.components.workable.workleft > 0 and (other.components.workable.action == ACTIONS.CHOP or other.components.workable.action == ACTIONS.MINE) then
            SpawnPrefab("collapse_small").Transform:SetPosition(other:GetPosition():Get())
            other.components.lootdropper:SetLoot({})
            other.components.workable:Destroy(inst)
        end
    end)
end

local function ontimerdone(inst, data)
    if data.name == "GroundPound" then
        inst.CanGroundPound = true
    end
end

local function OnCombatTarget(inst, data)
    if data.target then
        inst.num_food_cherrypicked = TUNING.BEARGER_STOLEN_TARGETS_FOR_AGRO - 1
        inst.components.locomotor.walkspeed = TUNING.BEARGER_ANGRY_WALK_SPEED
        data.target.BEARGER_OnDropItemFn = function(target, info)
            if inst.components.eater:CanEat(info.item) then
                if info.item:HasTag("honeyed") or math.random() < 1 then
                    inst.components.combat:SetTarget(nil)
                end
            end
        end
        inst:ListenForEvent("dropitem", data.target.BEARGER_OnDropItemFn, data.target)
    else
        inst.components.locomotor.walkspeed = TUNING.BEARGER_CALM_WALK_SPEED 
    end
end

local function SetStandState(inst, state)
    inst.StandState = string.lower(state)
end

local function IsStandState(inst, state)
    return inst.StandState == string.lower(state)
end

local function fn(Sim)
    local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	local sound = inst.entity:AddSoundEmitter()
	local shadow = inst.entity:AddDynamicShadow()
	shadow:SetSize(6, 3.5)    
    inst.Transform:SetFourFaced()  
    local s = 1
    trans:SetScale(s,s,s)
	MakeCharacterPhysics(inst, 1000, 1.5)
    inst.Physics:SetCollisionCallback(OnCollide)
	local minimap = inst.entity:AddMiniMapEntity()
    minimap:SetIcon( "friendlybearger.tex" )

    anim:SetBank("bearger")
    anim:SetBuild("bearger_build")
    anim:PlayAnimation("idle_loop", true)

	inst:AddTag("character")	
	inst:AddTag("notraptrigger")
	inst:AddTag("scarytoprey")
	inst:AddTag("companion")
	inst:AddTag("summonedbyplayer")
	inst:AddTag("irreplaceable")
	inst:AddTag("nosteal")
	inst:AddTag("kingdom")
	
	SetStandState(inst, "QUAD")--SetStandState(inst, "BI")
    inst.SetStandState = SetStandState
    inst.IsStandState = IsStandState
    inst.SeenBase = false
    inst.CanGroundPound = false
    inst.KilledPlayer = false
    inst.WorkEntities = WorkEntities
	
	inst:AddComponent("health")
    inst.components.health:SetMaxHealth(3000)
	inst.components.health:StartRegen(1,1)
	inst.components.health.canheal = false
	inst.components.health.canmurder = false
	inst.components.health.destroytime = 5
	if (IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(REIGN_OF_GIANTS)) then
		inst.components.health:SetAbsorptionAmount(0.20)
	else
		inst.components.health:SetAbsorbAmount(0.20)
	end

    inst:AddComponent("follower")
	inst.components.follower.maxfollowtime = 9999999
    inst.components.follower:AddLoyaltyTime(999999)	
	inst:AddComponent("followersitcommand")
	
	inst:AddComponent("timer")
	inst:AddComponent("thief")
    inst:AddComponent("inventory")
    inst:AddComponent("eater")

	inst:AddComponent("combat")
	inst.components.combat.hiteffectsymbol = "bearger_body"
    inst.components.combat:SetDefaultDamage(TUNING.DEERCLOPS_DAMAGE)
	inst.components.combat:SetRange(TUNING.BEARGER_ATTACK_RANGE, TUNING.BEARGER_MELEE_RANGE)
	inst.components.combat:SetAreaDamage(6, 0.8)
	inst.components.combat.notags = {"character"}
    inst.components.combat:SetAttackPeriod(2.5)	
    inst.components.combat:SetKeepTargetFunction(OnKeepTarget)
	inst.components.combat:SetRetargetFunction(1, NormalRetarget)
	inst.components.combat:SetHurtSound("dontstarve_DLC001/creatures/bearger/hurt")
	inst:ListenForEvent("killed", function(inst, data)
        if inst.components.combat and data and data.victim == inst.components.combat.target then
            inst.components.combat.target = nil
        end 
    end)

    inst:AddComponent("sleeper")
    inst.components.sleeper:SetResistance(4)
    inst.components.sleeper:SetSleepTest(ShouldSleep)
    inst.components.sleeper:SetWakeTest(ShouldWake)

    inst:AddComponent("lootdropper")
    inst.components.lootdropper:SetChanceLootTable("bearger")
	inst:AddComponent("norecipelootdrop")

    inst:AddComponent("inspectable")
    inst.components.inspectable:RecordViews()

    inst:AddComponent("knownlocations")
	
    inst:AddComponent("groundpounder")
	inst.components.groundpounder.noTags = {"character"}
    inst.components.groundpounder.destroyer = true
    inst.components.groundpounder.damageRings = 3
    inst.components.groundpounder.destructionRings = 4
    inst.components.groundpounder.numRings = 5

    MakeLargeBurnableCharacter(inst, "swap_fire")
    MakeHugeFreezableCharacter(inst, "bearger_body")

    inst:AddComponent("locomotor")
    inst.components.locomotor.walkspeed = TUNING.BEARGER_CALM_WALK_SPEED
    inst.components.locomotor.runspeed = TUNING.BEARGER_RUN_SPEED
    inst.components.locomotor:SetShouldRun(true)
	
	inst:AddComponent("inventoryitem")
    inst.components.inventoryitem.nobounce = true
    inst.components.inventoryitem.canbepickedup = true
	inst.components.inventoryitem.imagename = "friendlybearger"
    inst.components.inventoryitem.atlasname = "images/friendlybearger.xml"
	
	inst:DoPeriodicTask(1,function()
		if inst.components.follower and inst.components.follower.leader == nil and inst.components.followersitcommand:IsCurrentlyStaying() == false then	
			GetPlayer().components.leader:AddFollower(inst)
		elseif inst.components.follower and inst.components.follower.leader == GetPlayer() and inst.components.followersitcommand:IsCurrentlyStaying() == true then
			GetPlayer().components.leader:RemoveFollower(inst)
		end
	end)

    inst:SetStateGraph("SGfriendlybearger")
    local brain = require("brains/friendlybeargerbrain")
    inst:SetBrain(brain)
	
	inst:ListenForEvent("attacked", OnAttacked)
	inst:ListenForEvent("newcombattarget", OnNewTarget)
	inst:ListenForEvent("newcombattarget", OnCombatTarget)
    return inst
end

STRINGS.NAMES.FRIENDLYBEARGER = "Bearger"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.FRIENDLYBEARGER = "I knew it, he was looking for friends."
STRINGS.RECIPE_DESC.FRIENDLYBEARGER = "Summon the strongest of the giants as your minion."

return Prefab( "common/monsters/friendlybearger", fn, assets, prefabs) 