local assets =
{
	Asset("ANIM", "anim/deerclops_basic.zip"),
	Asset("ANIM", "anim/deerclops_actions.zip"),
	Asset("ANIM", "anim/deerclops_build.zip"),
	Asset("SOUND", "sound/deerclops.fsb"),
	Asset("IMAGE", "images/friendlydeerclops.tex"),
	Asset("ATLAS", "images/friendlydeerclops.xml"),
}

local prefabs =
{
	"meat",
	"deerclops_eyeball",
    "collapse_small",
    "icespike_fx_1",
    "icespike_fx_2",
    "icespike_fx_3",
    "icespike_fx_4",
}

local function OnKeepTarget(inst, target)
    return inst.components.combat:CanTarget(target) 
end

local function NormalRetarget(inst)
    local notags = {"FX", "NOCLICK","INLIMBO", "companion", "character", "player", "structure"}
    return FindEntity(inst, 16, 
        function(guy) 
            if inst.components.combat:CanTarget(guy) then
                return (guy:HasTag("monster") or guy:HasTag("hostile") or guy.components.combat.target == GetPlayer() or GetPlayer().components.combat.target == guy)
            end
    end, nil, notags)
end

local function ShouldSleep(inst)
    return false
end

local function ShouldWake(inst)
    return true
end

local function OnNewTarget(inst, data)
    inst.components.combat:ShareTarget(data.target, 30, function(dude) return dude:HasTag("summonedbyplayer") end, 15)
end

local function OnHitOther(inst, data)
    local other = data.target
    if other and other.components.freezable then
        other.components.freezable:AddColdness(2)
        other.components.freezable:SpawnShatterFX()
    end
end

local function OnAttacked(inst, data)
    local attacker = data.attacker
    inst.components.combat:SetTarget(attacker)
    inst.components.combat:ShareTarget(attacker, 30, function(dude) return dude:HasTag("summonedbyplayer") end, 15)
end

local function oncollide(inst, other)
    if other == nil or not other:HasTag("tree") then return end 
    local v1 = Vector3(inst.Physics:GetVelocity())
    if v1:LengthSq() < 1 then return end
    inst:DoTaskInTime(2*FRAMES, function()
        if other and other.components.workable and other.components.workable.workleft > 0 and (other.components.workable.action == ACTIONS.CHOP or other.components.workable.action == ACTIONS.MINE) then
            SpawnPrefab("collapse_small").Transform:SetPosition(other:GetPosition():Get())
            other.components.workable:Destroy(inst)
        end
    end)
end

local loot = {"meat", "meat", "meat", "meat", "meat", "meat", "meat", "meat", "deerclops_eyeball"}

local function fn(Sim)
    
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	local sound = inst.entity:AddSoundEmitter()
	local shadow = inst.entity:AddDynamicShadow()
    local s  = 1.65
    inst.Transform:SetScale(s,s,s)
	shadow:SetSize( 6, 3.5 )
    inst.Transform:SetFourFaced()
	MakeCharacterPhysics(inst, 1000, .5)
    inst.Physics:SetCollisionCallback(oncollide)
	local minimap = inst.entity:AddMiniMapEntity()
    minimap:SetIcon( "friendlydeerclops.tex" )
	
	inst:AddTag("character")	
	inst:AddTag("notraptrigger")
	inst:AddTag("scarytoprey")
	inst:AddTag("companion")
	inst:AddTag("summonedbyplayer")
	inst:AddTag("irreplaceable")
	inst:AddTag("nosteal")
	inst:AddTag("kingdom")

    anim:SetBank("deerclops")
    anim:SetBuild("deerclops_build")
    anim:PlayAnimation("idle_loop", true)

    inst:AddComponent("locomotor")
    inst.components.locomotor.walkspeed = 3  
    
    inst:SetStateGraph("SGfriendlydeerclops")

    MakeLargeBurnableCharacter(inst, "swap_fire")
    MakeHugeFreezableCharacter(inst, "deerclops_body")
    inst.components.freezable:SetResistance(6)

    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(2000)
	inst.components.health:StartRegen(1,1)
	inst.components.health.canheal = false
	inst.components.health.canmurder = false
	if (IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(REIGN_OF_GIANTS)) then
		inst.components.health:SetAbsorptionAmount(0.20)
	else
		inst.components.health:SetAbsorbAmount(0.20)
	end
	
	inst:AddComponent("follower")
	inst.components.follower.maxfollowtime = 9999999
    inst.components.follower:AddLoyaltyTime(999999)	
	inst:AddComponent("followersitcommand")
	
	inst:AddComponent("combat")
	inst.components.combat.hiteffectsymbol = "deerclops_body"
    inst.components.combat:SetDefaultDamage(TUNING.DEERCLOPS_DAMAGE)
	inst.components.combat:SetRange(TUNING.DEERCLOPS_ATTACK_RANGE)
	inst.components.combat:SetAreaDamage(TUNING.DEERCLOPS_AOE_RANGE, TUNING.DEERCLOPS_AOE_SCALE)
	inst.components.combat.notags = {"character"}
    inst.components.combat:SetAttackPeriod(2.5)	
    inst.components.combat:SetKeepTargetFunction(OnKeepTarget)
	inst.components.combat:SetRetargetFunction(1, NormalRetarget)
 
    inst:AddComponent("sleeper")
    inst.components.sleeper:SetResistance(4)
    inst.components.sleeper:SetSleepTest(ShouldSleep)
    inst.components.sleeper:SetWakeTest(ShouldWake)

    inst:AddComponent("lootdropper")
    inst.components.lootdropper:SetLoot(loot)
	inst:AddComponent("norecipelootdrop")

    inst:AddComponent("inspectable")
	inst.components.inspectable:RecordViews()
	
    inst:AddComponent("knownlocations")
	
	inst:AddComponent("inventoryitem")
    inst.components.inventoryitem.nobounce = true
    inst.components.inventoryitem.canbepickedup = true
	inst.components.inventoryitem.imagename = "friendlydeerclops"
    inst.components.inventoryitem.atlasname = "images/friendlydeerclops.xml"
	
	inst:DoPeriodicTask(1,function()
		if inst.components.follower and inst.components.follower.leader == nil and inst.components.followersitcommand:IsCurrentlyStaying() == false then	
			GetPlayer().components.leader:AddFollower(inst)
		elseif inst.components.follower and inst.components.follower.leader == GetPlayer() and inst.components.followersitcommand:IsCurrentlyStaying() == true then
			GetPlayer().components.leader:RemoveFollower(inst)
		end
	end)
	
	local brain = require "brains/friendlydeerclopsbrain"
    inst:SetBrain(brain)
    
    inst:ListenForEvent("attacked", OnAttacked)
    inst:ListenForEvent("onhitother", OnHitOther)
	inst:ListenForEvent("newcombattarget", OnNewTarget)
    return inst
end

STRINGS.NAMES.FRIENDLYDEERCLOPS = "Deerclops"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.FRIENDLYDEERCLOPS = "The king of winter!"
STRINGS.RECIPE_DESC.FRIENDLYDEERCLOPS = "Summon the king of winter as a servant!"

return Prefab( "common/monsters/friendlydeerclops", fn, assets, prefabs) 