require("brains/dragonflybrain")
require "stategraphs/SGdragonfly"

local assets =
{
	Asset("ANIM", "anim/dragonfly_build.zip"),
    Asset("ANIM", "anim/dragonfly_fire_build.zip"),
    Asset("ANIM", "anim/dragonfly_basic.zip"),
    Asset("ANIM", "anim/dragonfly_actions.zip"),
	Asset("IMAGE", "images/friendlydragonfly.tex"),
	Asset("ATLAS", "images/friendlydragonfly.xml"),
}

local prefabs =
{
    "lavaspit",
    "dragon_scales",
    "firesplash_fx",
    "tauntfire_fx",
    "attackfire_fx",
    "vomitfire_fx",
    "firering_fx",
    "collapse_small",
}

SetSharedLootTable( 'dragonfly',
{
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},
    {'dragon_scales',    1.00},
})

local BASE_TAGS = {"structure"}
local SEE_STRUCTURE_DIST = 20

local TARGET_DIST = 3

local function LeaveWorld(inst)
    inst:Remove()
end

local function NearPlayerBase(inst)
    local pt = inst:GetPosition()
    local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, SEE_STRUCTURE_DIST, BASE_TAGS)
    if #ents >= 2 then
        inst.SeenBase = true
        return true
    end
end

local function OnKeepTarget(inst, target)
    return inst.components.combat:CanTarget(target) 
end

local function NormalRetarget(inst)
    local notags = {"FX", "NOCLICK","INLIMBO", "companion", "character", "player", "structure"}
    return FindEntity(inst, 16, 
        function(guy) 
            if inst.components.combat:CanTarget(guy) then
                return (guy:HasTag("monster") or guy:HasTag("hostile") or guy.components.combat.target == GetPlayer() or GetPlayer().components.combat.target == guy)
            end
    end, nil, notags)
end

local function OnEntitySleep(inst)
    if ((not inst:NearPlayerBase() and inst.SeenBase and not inst.components.combat:TargetIs(GetPlayer()))
        or inst.components.sleeper:IsAsleep() 
        or inst.KilledPlayer)
        and not NearPlayerBase(inst) then
        --Dragonfly has seen your base and been lured off! Despawn.
        --Or the dragonfly has killed you, you've been punished enough.
        --Only applies if not currently at a base
        LeaveWorld(inst)
    elseif (not inst:NearPlayerBase() and not inst.SeenBase) 
        or (inst.components.combat:TargetIs(GetPlayer()) and not inst.KilledPlayer) then
        --Get back in there Dragonfly! You still have work to do.
        print("Porting Dragonfly to Player!")
        local init_pos = inst:GetPosition()
        local player_pos = GetPlayer():GetPosition()
        local angle = GetPlayer():GetAngleToPoint(init_pos)
        local offset = FindWalkableOffset(player_pos, angle*DEGREES, 30, 10)
        local pos = player_pos + offset
        
        if pos and distsq(player_pos, init_pos) > 1600 then
            --There's a crash if you teleport without the delay
            if not inst.components.combat:TargetIs(GetPlayer()) then
                inst.components.combat:SetTarget(nil)
            end
            inst:DoTaskInTime(.1, function() 
                inst.Transform:SetPosition(pos:Get())
            end)
        end
    elseif inst.shouldGoAway then
        LeaveWorld(inst)
    end
end

local function SetFlameOn(inst, flameon, newtarget, freeze)
    if flameon and not inst.flame_on then
        inst.flame_on = true
        if newtarget then
            inst.sg:GoToState("taunt_pre")
        end
    elseif not flameon and inst.flame_on then
        if freeze then
            inst.flame_on = false
            inst.Light:Enable(false)
            inst.components.propagator:StopSpreading()
            inst.AnimState:SetBuild("dragonfly_build")
            inst.fire_build = false
        elseif inst.components.combat and not inst.components.combat.target then
            inst.sg:GoToState("flameoff")
        end
    end
end

local function OnNewTarget(inst, data)
    inst.components.combat:ShareTarget(data.target, 30, function(dude) return dude:HasTag("summonedbyplayer") end, 15)
end

local function OnAttacked(inst, data)
    local attacker = data.attacker
    inst.components.combat:SetTarget(attacker)
    inst.components.combat:ShareTarget(attacker, 30, function(dude) return dude:HasTag("summonedbyplayer") end, 15)
end

local function OnFreeze(inst)
    inst.SoundEmitter:KillSound("flying")
    inst:ClearBufferedAction()
    SetFlameOn(inst, false, nil, true)
end

local function OnUnfreeze(inst)
    inst.recently_frozen = true
    inst.components.locomotor.walkspeed = 2
    inst.SoundEmitter:PlaySound("dontstarve_DLC001/creatures/dragonfly/fly", "flying")
    inst.components.combat:SetTarget(nil)

    inst:DoTaskInTime(5, function(inst) 
        inst.recently_frozen = false
        inst.components.locomotor.walkspeed = 4
        inst.spit_interval = math.random(20,30)
    end)
end

local function ShouldSleep(inst)
    return false
end

local function ShouldWake(inst)
    return true
end

local function OnCollide(inst, other)
    if other:HasTag("burnt") then
        -- local v1 = Vector3(inst.Physics:GetVelocity())
        -- if v1:LengthSq() < 1 then return end

        inst:DoTaskInTime(2*FRAMES, function()
            if other and other.components.workable and other.components.workable.workleft > 0 and (other.components.workable.action == ACTIONS.CHOP or other.components.workable.action == ACTIONS.MINE) then
                SpawnPrefab("collapse_small").Transform:SetPosition(other:GetPosition():Get())
                other.components.lootdropper:SetLoot({})
                other.components.workable:Destroy(inst)
            end
        end)
    end
end

local function OnSleep(inst)
    inst.SoundEmitter:KillSound("flying")
    inst.SoundEmitter:KillSound("vomitrumble")
    inst.SoundEmitter:KillSound("sleep")
    inst.SoundEmitter:KillSound("fireflying")
end

local function OnRemove(inst)
    inst.SoundEmitter:KillSound("flying")
    inst.SoundEmitter:KillSound("vomitrumble")
    inst.SoundEmitter:KillSound("sleep")
    inst.SoundEmitter:KillSound("fireflying")
end

local function OnKill(inst, data)
    if inst.components.combat and data and data.victim == inst.components.combat.target then
        inst.components.combat.target = nil
        inst.last_kill_time = GetTime()
    end 

    if data and data.victim == GetPlayer() then
        inst.KilledPlayer = true
    end
end

local function fn(Sim)
    local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	local sound = inst.entity:AddSoundEmitter()
	local shadow = inst.entity:AddDynamicShadow()
	shadow:SetSize(6, 3.5)    
    inst.Transform:SetFourFaced()
    inst.SoundEmitter:PlaySound("dontstarve_DLC001/creatures/dragonfly/fly", "flying")
    inst.Transform:SetScale(1.3,1.3,1.3)  
	MakeCharacterPhysics(inst, 500, 1.4)
    inst.Physics:SetCollisionCallback(OnCollide)
    inst.OnEntitySleep = OnSleep 
    inst.OnRemoveEntity = OnRemove
	local minimap = inst.entity:AddMiniMapEntity()
    minimap:SetIcon( "friendlydragonfly.tex" )

    anim:SetBank("dragonfly")
    anim:SetBuild("dragonfly_build")
    anim:PlayAnimation("idle", true)

	inst:AddTag("character")	
	inst:AddTag("notraptrigger")
	inst:AddTag("scarytoprey")
	inst:AddTag("companion")
	inst:AddTag("summonedbyplayer")
	inst:AddTag("irreplaceable")
	inst:AddTag("nosteal")
	inst:AddTag("kingdom")	
	
    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(2750)
	inst.components.health:StartRegen(1,1)
	inst.components.health.canheal = false
	inst.components.health.canmurder = false
	inst.components.health.destroytime = 5
    inst.components.health.fire_damage_scale = 0
	if (IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(REIGN_OF_GIANTS)) then
		inst.components.health:SetAbsorptionAmount(0.20)
	else
		inst.components.health:SetAbsorbAmount(0.20)
	end

    inst:AddComponent("groundpounder")
    inst.components.groundpounder.numRings = 2
    --inst.components.groundpounder.burner = true
    inst.components.groundpounder.groundpoundfx = "firesplash_fx"
    inst.components.groundpounder.groundpounddamagemult = .5
    inst.components.groundpounder.groundpoundringfx = "firering_fx"
	inst.components.groundpounder.noTags = {"character"}
    
	inst:AddComponent("combat")
	inst.components.combat.hiteffectsymbol = "dragonfly_body"
    inst.components.combat:SetDefaultDamage(TUNING.DRAGONFLY_DAMAGE)
	inst.components.combat:SetRange(4)
	inst.components.combat:SetAreaDamage(6, 0.8)
	inst.components.combat.notags = {"character"}
    inst.components.combat:SetAttackPeriod(2.5)	
    inst.components.combat:SetKeepTargetFunction(OnKeepTarget)
	inst.components.combat:SetRetargetFunction(1, NormalRetarget)
	inst.components.combat.battlecryenabled = false
	inst.components.combat:SetHurtSound("dontstarve_DLC001/creatures/dragonfly/hurt")
    inst.flame_on = false
    inst.KilledPlayer = false
    inst:ListenForEvent("killed", OnKill)
    inst:ListenForEvent("losttarget", function(inst) SetFlameOn(inst, false) end)
    inst:ListenForEvent("giveuptarget", function(inst) SetFlameOn(inst, false) end)
    inst:ListenForEvent("newcombattarget", function(inst, data) if data.target ~= nil then SetFlameOn(inst, true, true) end end)
    inst.SetFlameOn = SetFlameOn

    MakeLargePropagator(inst)
    inst.components.propagator.decayrate = 0

    local light = inst.entity:AddLight()
    inst.Light:Enable(false)
    inst.Light:SetRadius(2)
    inst.Light:SetFalloff(0.5)
    inst.Light:SetIntensity(.75)
    inst.Light:SetColour(235/255,121/255,12/255)
 
    inst:AddComponent("sleeper")
    inst.components.sleeper:SetResistance(4)
    inst.components.sleeper:SetSleepTest(ShouldSleep)
    inst.components.sleeper:SetWakeTest(ShouldWake)
    inst.playsleepsound = false
    inst.shouldGoAway = false

	inst:AddComponent("follower")
	inst.components.follower.maxfollowtime = 9999999
    inst.components.follower:AddLoyaltyTime(999999)	
	inst:AddComponent("followersitcommand")

    inst:AddComponent("lootdropper")
    inst.components.lootdropper:SetChanceLootTable("dragonfly")
    inst:AddComponent("norecipelootdrop")
	
    inst:AddComponent("inspectable")
    inst.components.inspectable:RecordViews()

    inst:AddComponent("knownlocations")
    inst:AddComponent("inventory")

    inst:ListenForEvent("attacked", OnAttacked)
    inst:ListenForEvent("entitysleep", OnEntitySleep)

    MakeHugeFreezableCharacter(inst, "dragonfly_body")
    inst.components.freezable.wearofftime = 1.5
    inst:ListenForEvent("freeze", OnFreeze)
    inst:ListenForEvent("unfreeze", OnUnfreeze)

    inst.SeenBase = false
    inst.NearPlayerBase = NearPlayerBase
    inst.last_spit_time = nil
    inst.last_target_spit_time = nil
    inst.spit_interval = math.random(20,30)
    inst.num_targets_vomited = 0
    inst.num_ashes_eaten = 0

    inst:AddComponent("locomotor")
    inst.components.locomotor.walkspeed = 4
	
	inst:AddComponent("inventoryitem")
    inst.components.inventoryitem.nobounce = true
    inst.components.inventoryitem.canbepickedup = true
	inst.components.inventoryitem.imagename = "friendlydragonfly"
    inst.components.inventoryitem.atlasname = "images/friendlydragonfly.xml"
	
	inst:DoPeriodicTask(1,function()
		if inst.components.follower and inst.components.follower.leader == nil and inst.components.followersitcommand:IsCurrentlyStaying() == false then	
			GetPlayer().components.leader:AddFollower(inst)
		elseif inst.components.follower and inst.components.follower.leader == GetPlayer() and inst.components.followersitcommand:IsCurrentlyStaying() == true then
			GetPlayer().components.leader:RemoveFollower(inst)
		end
	end)

    inst:SetStateGraph("SGfriendlydragonfly")
    local brain = require("brains/friendlydragonflybrain")
    inst:SetBrain(brain)
	
	inst:ListenForEvent("attacked", OnAttacked)
	inst:ListenForEvent("newcombattarget", OnNewTarget)
    return inst
end

STRINGS.NAMES.FRIENDLYDRAGONFLY = "Dragonfly"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.FRIENDLYDRAGONFLY = "There you are salamander!"
STRINGS.RECIPE_DESC.FRIENDLYDRAGONFLY = "Summon the power of a dragon!"

return Prefab( "common/monsters/friendlydragonfly", fn, assets, prefabs)