local assets =
{
	Asset("ANIM", "anim/goosemoose_build.zip"),
    Asset("ANIM", "anim/goosemoose_basic.zip"),
    Asset("ANIM", "anim/goosemoose_actions.zip"),
    --Asset("SOUND", "sound/moose.fsb"),
	Asset("IMAGE", "images/friendlymoose.tex"),
	Asset("ATLAS", "images/friendlymoose.xml"),
}

local prefabs =
{
    "mooseegg",
    "mossling",
    "goose_feather",
    "drumstick",
}

local MOOSE_SCALE = 1.55

SetSharedLootTable( 'moose',
{
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},
    {'drumstick',        1.00},
    {'drumstick',        1.00},
    {'goose_feather',    1.00},
    {'goose_feather',    1.00},
    {'goose_feather',    1.00},
    {'goose_feather',    0.33},
    {'goose_feather',    0.33},
})

local BASE_TAGS = {"structure"}
local SEE_STRUCTURE_DIST = 20

local TARGET_DIST = 10
local LOSE_TARGET_DIST = 30

local function LeaveWorld(inst)
    inst:Remove()
end

local function NearPlayerBase(inst)
    local pt = inst:GetPosition()
    local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, SEE_STRUCTURE_DIST, BASE_TAGS)
    if #ents >= 2 then
        inst.SeenBase = true
        return true
    end
end

local function OnKeepTarget(inst, target)
    return inst.components.combat:CanTarget(target) 
end

local function NormalRetarget(inst)
    local notags = {"FX", "NOCLICK","INLIMBO", "companion", "character", "player", "structure"}
    return FindEntity(inst, 16, 
        function(guy) 
            if inst.components.combat:CanTarget(guy) then
                return (guy:HasTag("monster") or guy:HasTag("hostile") or guy.components.combat.target == GetPlayer() or GetPlayer().components.combat.target == guy)
            end
    end, nil, notags)
end

local function ShouldSleep(inst)
    return false
end

local function ShouldWake(inst)
    return true
end

local function OnNewTarget(inst, data)
    inst.components.combat:ShareTarget(data.target, 30, function(dude) return dude:HasTag("summonedbyplayer") end, 15)
end

local function OnAttacked(inst, data)
    local attacker = data.attacker
    inst.components.combat:SetTarget(attacker)
    inst.components.combat:ShareTarget(attacker, 30, function(dude) return dude:HasTag("summonedbyplayer") end, 15)
end

local function OnCollide(inst, other)
end

local function OnSave(inst, data)
    data.WantsToLayEgg = inst.WantsToLayEgg
    data.CanDisarm = inst.CanDisarm
    data.shouldGoAway = inst.shouldGoAway
end

local function OnLoad(inst, data)
    if data.WantsToLayEgg then 
        inst.WantsToLayEgg = data.WantsToLayEgg
    end
    if data.CanDisarm then 
        inst.CanDisarm = data.CanDisarm
    end
    inst.shouldGoAway = data.shouldGoAway or false
end

local function ontimerdone(inst, data)
    if data.name == "WantsToLayEgg" then
        inst.WantsToLayEgg = true
    end

    if data.name == "DisarmCooldown" then
        inst.CanDisarm = true
    end
end

local function fn(Sim)
    local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	local sound = inst.entity:AddSoundEmitter()
	local shadow = inst.entity:AddDynamicShadow()
    local s = MOOSE_SCALE
    trans:SetScale(s,s,s)
	shadow:SetSize(6, 2.75)
	MakeCharacterPhysics(inst, 5000, 1)
    inst.Physics:SetCollisionCallback(OnCollide)
    inst.Transform:SetFourFaced()
	local minimap = inst.entity:AddMiniMapEntity()
    minimap:SetIcon( "friendlymoose.tex" )

    anim:SetBank("goosemoose")
    anim:SetBuild("goosemoose_build")
    anim:PlayAnimation("idle", true)

    inst:AddTag("character")	
	inst:AddTag("notraptrigger")
	inst:AddTag("scarytoprey")
	inst:AddTag("companion")
	inst:AddTag("summonedbyplayer")
	inst:AddTag("irreplaceable")
	inst:AddTag("nosteal")
	inst:AddTag("kingdom")
	
	inst:AddComponent("health")
    inst.components.health:SetMaxHealth(3000)
	inst.components.health:StartRegen(1,1)
	inst.components.health.destroytime = 3
	inst.components.health.canheal = false
	inst.components.health.canmurder = false
	if (IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(REIGN_OF_GIANTS)) then
		inst.components.health:SetAbsorptionAmount(0.20)
	else
		inst.components.health:SetAbsorbAmount(0.20)
	end
	
	inst:AddComponent("combat")
	inst.components.combat.hiteffectsymbol = "goosemoose_body"
    inst.components.combat:SetDefaultDamage(250)
	inst.components.combat:SetRange(TUNING.MOOSE_ATTACK_RANGE)
	inst.components.combat.notags = {"character"}
    inst.components.combat:SetAttackPeriod(2.5)	
    inst.components.combat:SetKeepTargetFunction(OnKeepTarget)
	inst.components.combat:SetRetargetFunction(1, NormalRetarget)
	inst.components.combat:SetHurtSound("dontstarve_DLC001/creatures/moose/hurt")
	
	inst:AddComponent("follower")
	inst.components.follower.maxfollowtime = 9999999
    inst.components.follower:AddLoyaltyTime(999999)	
	inst:AddComponent("followersitcommand")
 
    inst:AddComponent("sleeper")
    inst.shouldGoAway = false
    inst.components.sleeper:SetSleepTest(ShouldSleep)
    inst.components.sleeper:SetWakeTest(ShouldWake)

    inst:AddComponent("lootdropper")
    inst.components.lootdropper:SetChanceLootTable('moose')  
    inst:AddComponent("norecipelootdrop")
		
    inst:AddComponent("inspectable")
    inst.components.inspectable:RecordViews()

    inst:AddComponent("named")
    inst.components.named.possiblenames = {STRINGS.NAMES["MOOSE1"], STRINGS.NAMES["MOOSE2"]}
    inst.components.named:PickNewName()
    inst:DoPeriodicTask(5, function(inst)
        inst.components.named:PickNewName()
    end)

    inst:AddComponent("knownlocations")
    inst:AddComponent("inventory")
    inst:AddComponent("entitytracker")
    inst:AddComponent("timer")
    inst:AddComponent("eater")

    MakeLargeBurnableCharacter(inst, "swap_fire")
    MakeHugeFreezableCharacter(inst, "goosemoose_body")

    inst.WantsToLayEgg = false
    inst.CanDisarm = false

    inst:AddComponent("locomotor")
    inst.components.locomotor.walkspeed = TUNING.MOOSE_WALK_SPEED
    inst.components.locomotor.runspeed = TUNING.MOOSE_RUN_SPEED

    inst:SetStateGraph("SGfriendlymoose")
    local brain = require("brains/friendlymoosebrain")
    inst:SetBrain(brain)
	
	inst:AddComponent("inventoryitem")
    inst.components.inventoryitem.nobounce = true
    inst.components.inventoryitem.canbepickedup = true
	inst.components.inventoryitem.imagename = "friendlymoose"
    inst.components.inventoryitem.atlasname = "images/friendlymoose.xml"
	
	inst:DoPeriodicTask(1,function()
		if inst.components.follower and inst.components.follower.leader == nil and inst.components.followersitcommand:IsCurrentlyStaying() == false then	
			GetPlayer().components.leader:AddFollower(inst)
		elseif inst.components.follower and inst.components.follower.leader == GetPlayer() and inst.components.followersitcommand:IsCurrentlyStaying() == true then
			GetPlayer().components.leader:RemoveFollower(inst)
		end
	end)
	
	inst:ListenForEvent("attacked", OnAttacked)
	inst:ListenForEvent("newcombattarget", OnNewTarget)
    return inst
end

STRINGS.NAMES.FRIENDLYMOOSE = "Moose"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.FRIENDLYMOOSE = "She thinks that I'm her daddy."
STRINGS.RECIPE_DESC.FRIENDLYMOOSE = "Summon the moose!"

return Prefab("common/monsters/friendlymoose", fn, assets, prefabs) 